@@ -4,6 +4,7 @@ use nom::{branch::alt, IResult};
 pub mod core;
 
 pub mod bodystructure;
+pub mod gmail;
 pub mod rfc2087;
 pub mod rfc3501;
 pub mod rfc4315;
@@ -16,7 +17,7 @@ pub mod rfc7162;
 #[cfg(test)]
 mod tests;
 
-pub fn parse_response(msg: &[u8]) -> ParseResult {
+pub fn parse_response(msg: &[u8]) -> ParseResult<'_> {
     alt((
         rfc3501::continue_req,
         rfc3501::response_data,
