@@ -11,7 +11,7 @@ use nom::{
     branch::alt,
     bytes::streaming::{tag, tag_no_case, take_while, take_while1},
     character::streaming::char,
-    combinator::{map, map_res, opt, recognize},
+    combinator::{map, map_res, opt, recognize, value},
     multi::{many0, many1},
     sequence::{delimited, pair, preceded, terminated, tuple},
     IResult,
@@ -25,6 +25,8 @@ use crate::{
     types::*,
 };
 
+use super::gmail;
+
 pub mod body;
 pub mod body_structure;
 
@@ -69,7 +71,7 @@ fn flag_extension(i: &[u8]) -> IResult<&[u8], &str> {
     )(i)
 }
 
-fn flag(i: &[u8]) -> IResult<&[u8], &str> {
+pub(crate) fn flag(i: &[u8]) -> IResult<&[u8], &str> {
     alt((flag_extension, atom))(i)
 }
 
@@ -243,24 +245,50 @@ fn mailbox_data_exists(i: &[u8]) -> IResult<&[u8], MailboxDatum> {
     )(i)
 }
 
+fn name_attribute(i: &[u8]) -> IResult<&[u8], NameAttribute> {
+    alt((
+        // RFC 3501
+        value(NameAttribute::NoInferiors, tag_no_case(b"\\Noinferiors")),
+        value(NameAttribute::NoSelect, tag_no_case(b"\\Noselect")),
+        value(NameAttribute::Marked, tag_no_case(b"\\Marked")),
+        value(NameAttribute::Unmarked, tag_no_case(b"\\Unmarked")),
+        // RFC 6154
+        value(NameAttribute::All, tag_no_case(b"\\All")),
+        value(NameAttribute::Archive, tag_no_case(b"\\Archive")),
+        value(NameAttribute::Drafts, tag_no_case(b"\\Drafts")),
+        value(NameAttribute::Flagged, tag_no_case(b"\\Flagged")),
+        value(NameAttribute::Junk, tag_no_case(b"\\Junk")),
+        value(NameAttribute::Sent, tag_no_case(b"\\Sent")),
+        value(NameAttribute::Trash, tag_no_case(b"\\Trash")),
+        // Extensions not supported by this crate
+        map(
+            map_res(
+                recognize(pair(tag(b"\\"), take_while(is_atom_char))),
+                from_utf8,
+            ),
+            |s| NameAttribute::Extension(Cow::Borrowed(s)),
+        ),
+    ))(i)
+}
+
 #[allow(clippy::type_complexity)]
-fn mailbox_list(i: &[u8]) -> IResult<&[u8], (Vec<Cow<str>>, Option<&str>, &str)> {
+fn mailbox_list(i: &[u8]) -> IResult<&[u8], (Vec<NameAttribute>, Option<&str>, &str)> {
     map(
         tuple((
-            flag_list,
+            parenthesized_list(name_attribute),
             tag(b" "),
             alt((map(quoted_utf8, Some), map(nil, |_| None))),
             tag(b" "),
             mailbox,
         )),
-        |(flags, _, delimiter, _, name)| (flags, delimiter, name),
+        |(name_attributes, _, delimiter, _, name)| (name_attributes, delimiter, name),
     )(i)
 }
 
 fn mailbox_data_list(i: &[u8]) -> IResult<&[u8], MailboxDatum> {
     map(preceded(tag_no_case("LIST "), mailbox_list), |data| {
         MailboxDatum::List {
-            flags: data.0,
+            name_attributes: data.0,
             delimiter: data.1.map(Cow::Borrowed),
             name: Cow::Borrowed(data.2),
         }
@@ -270,7 +298,7 @@ fn mailbox_data_list(i: &[u8]) -> IResult<&[u8], MailboxDatum> {
 fn mailbox_data_lsub(i: &[u8]) -> IResult<&[u8], MailboxDatum> {
     map(preceded(tag_no_case("LSUB "), mailbox_list), |data| {
         MailboxDatum::List {
-            flags: data.0,
+            name_attributes: data.0,
             delimiter: data.1.map(Cow::Borrowed),
             name: Cow::Borrowed(data.2),
         }
@@ -335,6 +363,7 @@ fn mailbox_data(i: &[u8]) -> IResult<&[u8], MailboxDatum> {
         mailbox_data_status,
         mailbox_data_recent,
         mailbox_data_search,
+        gmail::mailbox_data_gmail_labels,
         rfc5256::mailbox_data_sort,
     ))(i)
 }
@@ -529,6 +558,7 @@ fn msg_att(i: &[u8]) -> IResult<&[u8], AttributeValue> {
         msg_att_rfc822_size,
         msg_att_rfc822_text,
         msg_att_uid,
+        gmail::msg_att_gmail_labels,
     ))(i)
 }
 
@@ -573,6 +603,13 @@ fn resp_text(i: &[u8]) -> IResult<&[u8], (Option<ResponseCode>, Option<&str>)> {
     })(i)
 }
 
+// an response-text if it is at the end of a response. Empty text is then allowed without the normally needed trailing space.
+fn trailing_resp_text(i: &[u8]) -> IResult<&[u8], (Option<ResponseCode>, Option<&str>)> {
+    map(opt(tuple((tag(b" "), resp_text))), |resptext| {
+        resptext.map(|(_, tuple)| tuple).unwrap_or((None, None))
+    })(i)
+}
+
 // continue-req    = "+" SP (resp-text / base64) CRLF
 pub(crate) fn continue_req(i: &[u8]) -> IResult<&[u8], Response> {
     // Some servers do not send the space :/
@@ -596,11 +633,10 @@ pub(crate) fn response_tagged(i: &[u8]) -> IResult<&[u8], Response> {
             imap_tag,
             tag(b" "),
             status,
-            tag(b" "),
-            resp_text,
+            trailing_resp_text,
             tag(b"\r\n"),
         )),
-        |(tag, _, status, _, text, _)| Response::Done {
+        |(tag, _, status, text, _)| Response::Done {
             tag,
             status,
             code: text.0,
@@ -617,14 +653,13 @@ pub(crate) fn response_tagged(i: &[u8]) -> IResult<&[u8], Response> {
 // resp-cond-state = ("OK" / "NO" / "BAD") SP resp-text
 //                     ; Status condition
 fn resp_cond(i: &[u8]) -> IResult<&[u8], Response> {
-    map(
-        tuple((status, tag(b" "), resp_text)),
-        |(status, _, text)| Response::Data {
+    map(tuple((status, trailing_resp_text)), |(status, text)| {
+        Response::Data {
             status,
             code: text.0,
             information: text.1.map(Cow::Borrowed),
-        },
-    )(i)
+        }
+    })(i)
 }
 
 // response-data   = "*" SP (resp-cond-state / resp-cond-bye /
