@@ -11,6 +11,44 @@ fn test_mailbox_data_response() {
     }
 }
 
+/// Test that the name attributes in [RFC 3501 Section 7.2.2](https://datatracker.ietf.org/doc/html/rfc3501#section-7.2.2)
+/// and extensions can be parsed.
+#[test]
+fn test_name_attributes() {
+    match parse_response(
+        b"* LIST (\\Noinferiors \\Noselect \\Marked \\Unmarked \\All \\Archive \\Drafts \\Flagged \\Junk \\Sent \\Trash \\Foobar) \".\" INBOX.Tests\r\n",
+    ) {
+        Ok((
+            _,
+            Response::MailboxData(MailboxDatum::List {
+                name_attributes, ..
+            }),
+        )) => {
+            assert_eq!(
+                name_attributes,
+                vec![
+                    // RFC 3501
+                    NameAttribute::NoInferiors,
+                    NameAttribute::NoSelect,
+                    NameAttribute::Marked,
+                    NameAttribute::Unmarked,
+                    // RFC 6154
+                    NameAttribute::All,
+                    NameAttribute::Archive,
+                    NameAttribute::Drafts,
+                    NameAttribute::Flagged,
+                    NameAttribute::Junk,
+                    NameAttribute::Sent,
+                    NameAttribute::Trash,
+                    // Extensions not supported by this crate
+                    NameAttribute::Extension(Cow::Borrowed("\\Foobar")),
+                ]
+            );
+        }
+        rsp => panic!("unexpected response {:?}", rsp),
+    }
+}
+
 #[test]
 fn test_number_overflow() {
     match parse_response(b"* 2222222222222222222222222222222222222222222C\r\n") {
@@ -371,7 +409,7 @@ fn test_flags() {
 fn test_vanished() {
     match parse_response(b"* VANISHED (EARLIER) 1,2,3:8\r\n") {
         Ok((_, Response::Vanished { earlier, uids })) => {
-            assert_eq!(earlier, true);
+            assert!(earlier);
             assert_eq!(uids.len(), 3);
             let v = &uids[0];
             assert_eq!(*v.start(), 1);
@@ -388,7 +426,7 @@ fn test_vanished() {
 
     match parse_response(b"* VANISHED 1,2,3:8,10\r\n") {
         Ok((_, Response::Vanished { earlier, uids })) => {
-            assert_eq!(earlier, false);
+            assert!(!earlier);
             assert_eq!(uids.len(), 4);
         }
         rsp => panic!("Unexpected response: {:?}", rsp),
@@ -396,7 +434,7 @@ fn test_vanished() {
 
     match parse_response(b"* VANISHED (EARLIER) 1\r\n") {
         Ok((_, Response::Vanished { earlier, uids })) => {
-            assert_eq!(earlier, true);
+            assert!(earlier);
             assert_eq!(uids.len(), 1);
             assert_eq!(uids[0].clone().collect::<Vec<u32>>(), vec![1]);
         }
@@ -405,7 +443,7 @@ fn test_vanished() {
 
     match parse_response(b"* VANISHED 1\r\n") {
         Ok((_, Response::Vanished { earlier, uids })) => {
-            assert_eq!(earlier, false);
+            assert!(!earlier);
             assert_eq!(uids.len(), 1);
         }
         rsp => panic!("Unexpected response: {:?}", rsp),
@@ -540,3 +578,29 @@ fn test_parsing_of_quota_capability_in_login_response() {
         rsp => panic!("unexpected response {:?}", rsp),
     }
 }
+
+#[test]
+fn test_parsing_of_bye_response() {
+    match parse_response(b"* BYE\r\n") {
+        Ok((
+            _,
+            Response::Data {
+                status: Status::Bye,
+                code: None,
+                information: None,
+            },
+        )) => {}
+        rsp => panic!("unexpected response {:?}", rsp),
+    };
+    match parse_response(b"* BYE Autologout; idle for too long\r\n") {
+        Ok((
+            _,
+            Response::Data {
+                status: Status::Bye,
+                code: None,
+                information: Some(Cow::Borrowed("Autologout; idle for too long")),
+            },
+        )) => {}
+        rsp => panic!("unexpected response {:?}", rsp),
+    };
+}
