@@ -46,7 +46,7 @@ pub enum Response<'a> {
 }
 
 impl<'a> Response<'a> {
-    pub fn from_bytes(buf: &'a [u8]) -> crate::ParseResult {
+    pub fn from_bytes(buf: &'a [u8]) -> crate::ParseResult<'a> {
         crate::parser::parse_response(buf)
     }
 
@@ -201,7 +201,7 @@ pub enum MailboxDatum<'a> {
     Exists(u32),
     Flags(Vec<Cow<'a, str>>),
     List {
-        flags: Vec<Cow<'a, str>>,
+        name_attributes: Vec<NameAttribute<'a>>,
         delimiter: Option<Cow<'a, str>>,
         name: Cow<'a, str>,
     },
@@ -220,6 +220,7 @@ pub enum MailboxDatum<'a> {
         mailbox: Cow<'a, str>,
         values: Vec<Cow<'a, str>>,
     },
+    GmailLabels(Vec<Cow<'a, str>>),
 }
 
 impl<'a> MailboxDatum<'a> {
@@ -230,11 +231,14 @@ impl<'a> MailboxDatum<'a> {
                 MailboxDatum::Flags(flags.into_iter().map(to_owned_cow).collect())
             }
             MailboxDatum::List {
-                flags,
+                name_attributes,
                 delimiter,
                 name,
             } => MailboxDatum::List {
-                flags: flags.into_iter().map(to_owned_cow).collect(),
+                name_attributes: name_attributes
+                    .into_iter()
+                    .map(|named_attribute| named_attribute.into_owned())
+                    .collect(),
                 delimiter: delimiter.map(to_owned_cow),
                 name: to_owned_cow(name),
             },
@@ -257,6 +261,9 @@ impl<'a> MailboxDatum<'a> {
                     values: values.into_iter().map(to_owned_cow).collect(),
                 }
             }
+            MailboxDatum::GmailLabels(labels) => {
+                MailboxDatum::GmailLabels(labels.into_iter().map(to_owned_cow).collect())
+            }
         }
     }
 }
@@ -290,6 +297,8 @@ pub enum Attribute {
     Rfc822Size,
     Rfc822Text,
     Uid,
+    /// https://developers.google.com/gmail/imap/imap-extensions#access_to_gmail_labels_x-gm-labels
+    GmailLabels,
 }
 
 #[derive(Debug, Eq, PartialEq)]
@@ -324,6 +333,8 @@ pub enum AttributeValue<'a> {
     Rfc822Size(u32),
     Rfc822Text(Option<Cow<'a, [u8]>>),
     Uid(u32),
+    /// https://developers.google.com/gmail/imap/imap-extensions#access_to_gmail_labels_x-gm-labels
+    GmailLabels(Vec<Cow<'a, str>>),
 }
 
 impl<'a> AttributeValue<'a> {
@@ -350,6 +361,9 @@ impl<'a> AttributeValue<'a> {
             AttributeValue::Rfc822Size(v) => AttributeValue::Rfc822Size(v),
             AttributeValue::Rfc822Text(v) => AttributeValue::Rfc822Text(v.map(to_owned_cow)),
             AttributeValue::Uid(v) => AttributeValue::Uid(v),
+            AttributeValue::GmailLabels(v) => {
+                AttributeValue::GmailLabels(v.into_iter().map(to_owned_cow).collect())
+            }
         }
     }
 }
@@ -560,12 +574,18 @@ fn body_param_owned<'a>(v: BodyParams<'a>) -> BodyParams<'static> {
     })
 }
 
+/// An RFC 2822 envelope
+///
+/// See https://datatracker.ietf.org/doc/html/rfc2822#section-3.6 for more details.
 #[derive(Debug, Eq, PartialEq)]
 pub struct Envelope<'a> {
     pub date: Option<Cow<'a, [u8]>>,
     pub subject: Option<Cow<'a, [u8]>>,
+    /// Author of the message; mailbox responsible for writing the message
     pub from: Option<Vec<Address<'a>>>,
+    /// Mailbox of the agent responsible for the message's transmission
     pub sender: Option<Vec<Address<'a>>>,
+    /// Mailbox that the author of the message suggests replies be sent to
     pub reply_to: Option<Vec<Address<'a>>>,
     pub to: Option<Vec<Address<'a>>>,
     pub cc: Option<Vec<Address<'a>>>,
@@ -705,6 +725,118 @@ impl<'a> BodyExtMPart<'a> {
     }
 }
 
+/// The name attributes are returned as part of a LIST response described in
+/// [RFC 3501 section 7.2.2](https://tools.ietf.org/html/rfc3501#section-7.2.2).
+///
+/// This enumeration additional includes values from the extension Special-Use
+/// Mailboxes [RFC 6154 section 2](https://tools.ietf.org/html/rfc6154#section-2).
+#[derive(Debug, Eq, PartialEq, Clone)]
+#[non_exhaustive]
+pub enum NameAttribute<'a> {
+    /// From [RFC 3501 section 7.2.2](https://tools.ietf.org/html/rfc3501#section-7.2.2):
+    ///
+    /// > It is not possible for any child levels of hierarchy to exist
+    /// > under this name; no child levels exist now and none can be
+    /// > created in the future.
+    NoInferiors,
+    /// From [RFC 3501 section 7.2.2](https://tools.ietf.org/html/rfc3501#section-7.2.2):
+    ///
+    /// > It is not possible to use this name as a selectable mailbox.
+    NoSelect,
+    /// From [RFC 3501 section 7.2.2](https://tools.ietf.org/html/rfc3501#section-7.2.2):
+    ///
+    /// > The mailbox has been marked "interesting" by the server; the
+    /// > mailbox probably contains messages that have been added since
+    /// > the last time the mailbox was selected.
+    Marked,
+    /// From [RFC 3501 section 7.2.2](https://tools.ietf.org/html/rfc3501#section-7.2.2):
+    ///
+    /// > The mailbox does not contain any additional messages since the
+    /// > last time the mailbox was selected.
+    Unmarked,
+    /// From [RFC 6154 section 2](https://tools.ietf.org/html/rfc6154#section-2):
+    ///
+    /// > This mailbox presents all messages in the user's message store.
+    /// > Implementations MAY omit some messages, such as, perhaps, those
+    /// > in \Trash and \Junk.  When this special use is supported, it is
+    /// > almost certain to represent a virtual mailbox.
+    All,
+    /// From [RFC 6154 section 2](https://tools.ietf.org/html/rfc6154#section-2):
+    ///
+    /// > This mailbox is used to archive messages.  The meaning of an
+    /// > "archival" mailbox is server-dependent; typically, it will be
+    /// > used to get messages out of the inbox, or otherwise keep them
+    /// > out of the user's way, while still making them accessible.
+    Archive,
+    /// From [RFC 6154 section 2](https://tools.ietf.org/html/rfc6154#section-2):
+    ///
+    /// > This mailbox is used to hold draft messages -- typically,
+    /// > messages that are being composed but have not yet been sent.  In
+    /// > some server implementations, this might be a virtual mailbox,
+    /// > containing messages from other mailboxes that are marked with
+    /// > the "\Draft" message flag.  Alternatively, this might just be
+    /// > advice that a client put drafts here.
+    Drafts,
+    /// From [RFC 6154 section 2](https://tools.ietf.org/html/rfc6154#section-2):
+    ///
+    /// > This mailbox presents all messages marked in some way as
+    /// > "important".  When this special use is supported, it is likely
+    /// > to represent a virtual mailbox collecting messages (from other
+    /// > mailboxes) that are marked with the "\Flagged" message flag.
+    Flagged,
+    /// From [RFC 6154 section 2](https://tools.ietf.org/html/rfc6154#section-2):
+    ///
+    /// > This mailbox is where messages deemed to be junk mail are held.
+    /// > Some server implementations might put messages here
+    /// > automatically.  Alternatively, this might just be advice to a
+    /// > client-side spam filter.
+    Junk,
+    /// From [RFC 6154 section 2](https://tools.ietf.org/html/rfc6154#section-2):
+    ///
+    /// > This mailbox is used to hold copies of messages that have been
+    /// > sent.  Some server implementations might put messages here
+    /// > automatically.  Alternatively, this might just be advice that a
+    /// > client save sent messages here.
+    Sent,
+    /// From [RFC 6154 section 2](https://tools.ietf.org/html/rfc6154#section-2)
+    ///
+    /// > This mailbox is used to hold messages that have been deleted or
+    /// > marked for deletion.  In some server implementations, this might
+    /// > be a virtual mailbox, containing messages from other mailboxes
+    /// > that are marked with the "\Deleted" message flag.
+    /// > Alternatively, this might just be advice that a client that
+    /// > chooses not to use the IMAP "\Deleted" model should use this as
+    /// > its trash location.  In server implementations that strictly
+    /// > expect the IMAP "\Deleted" model, this special use is likely not
+    /// > to be supported.
+    Trash,
+    /// A name attribute not defined in [RFC 3501 section 7.2.2](https://tools.ietf.org/html/rfc3501#section-7.2.2)
+    /// or any supported extension.
+    Extension(Cow<'a, str>),
+}
+
+impl<'a> NameAttribute<'a> {
+    pub fn into_owned(self) -> NameAttribute<'static> {
+        match self {
+            // RFC 3501
+            NameAttribute::NoInferiors => NameAttribute::NoInferiors,
+            NameAttribute::NoSelect => NameAttribute::NoSelect,
+            NameAttribute::Marked => NameAttribute::Marked,
+            NameAttribute::Unmarked => NameAttribute::Unmarked,
+            // RFC 6154
+            NameAttribute::All => NameAttribute::All,
+            NameAttribute::Archive => NameAttribute::Archive,
+            NameAttribute::Drafts => NameAttribute::Drafts,
+            NameAttribute::Flagged => NameAttribute::Flagged,
+            NameAttribute::Junk => NameAttribute::Junk,
+            NameAttribute::Sent => NameAttribute::Sent,
+            NameAttribute::Trash => NameAttribute::Trash,
+            // Extensions not supported by this crate
+            NameAttribute::Extension(s) => NameAttribute::Extension(to_owned_cow(s)),
+        }
+    }
+}
+
 // IMAP4 QUOTA extension (rfc2087)
 
 /// https://tools.ietf.org/html/rfc2087#section-3
@@ -785,3 +917,36 @@ impl<'a> QuotaRoot<'a> {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    /// Tests that the [`NameAttribute::into_owned`] method returns the
+    /// same value (the ownership should only change).
+    #[test]
+    fn test_name_attribute_into_owned() {
+        let name_attributes = [
+            // RFC 3501
+            NameAttribute::NoInferiors,
+            NameAttribute::NoSelect,
+            NameAttribute::Marked,
+            NameAttribute::Unmarked,
+            // RFC 6154
+            NameAttribute::All,
+            NameAttribute::Archive,
+            NameAttribute::Drafts,
+            NameAttribute::Flagged,
+            NameAttribute::Junk,
+            NameAttribute::Sent,
+            NameAttribute::Trash,
+            // Extensions not supported by this crate
+            NameAttribute::Extension(Cow::Borrowed("Foobar")),
+        ];
+
+        for name_attribute in name_attributes {
+            let owned_name_attribute = name_attribute.clone().into_owned();
+            assert_eq!(name_attribute, owned_name_attribute);
+        }
+    }
+}
